/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.crop.visual;

import javax.swing.JToolBar;
import nl.cloudfarming.client.crop.model.Classification;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.Variety;
import nl.cloudfarming.client.stock.crop.actions.AddCropDefinitionAction;
import nl.cloudfarming.client.stock.crop.actions.RemoveAction;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;

/**
 * Toolbar to store all actions possible inside the croplist. Registers itself
 * as lookuplistener of cropdefinition, classification and variety.
 *
 * @author Gerben Feenstra & Merijn Zengers
 */
class CropListToolBar extends JToolBar {

    private final Listener lookupListener = new Listener();
    private Lookup.Result<CropDefinition> cropDefLookupResult;
    private Lookup.Result<Classification> classificationLookupResult;
    private Lookup.Result<Variety> varietyLookupResult;
    private RemoveAction removeAction;
    private AddCropDefinitionAction addCropAction;
    private Lookup lookup;

    /**
     * Constructs toolbar, adds actions to it
     */
    public CropListToolBar(Lookup lookup) {
        this.lookup = lookup;
        initActions();
        initLookupResults();
    }

    private void initActions() {
        initRemoveAction();
        initAddCropAction();
    }

    private void initRemoveAction() {
        removeAction = new RemoveAction(lookup);
        removeAction.setEnabled(false);
        add(removeAction);
    }

    private void initAddCropAction() {
        addCropAction = new AddCropDefinitionAction();
        addCropAction.setEnabled(true);
        add(addCropAction);
    }

    private void initLookupResults() {
        initLookupResult(cropDefLookupResult, CropDefinition.class);
        initLookupResult(classificationLookupResult, Classification.class);
        initLookupResult(varietyLookupResult, Variety.class);
    }

    private void initLookupResult(Lookup.Result result, Class clazz) {
        result = lookup.lookupResult(clazz);
        result.addLookupListener(lookupListener);
    }

    /**
     * Listens to lookup changes if CropDefinition, classification or variety
     * are in the lookup it will enable the suitable actions
     */
    private class Listener implements LookupListener {

        @Override
        public void resultChanged(LookupEvent ev) {
            removeAction.setEnabled(false);

            Lookup.Result<Object> r = (Lookup.Result<Object>) ev.getSource();
            for (Object o : r.allInstances()) {
                if (o instanceof CropDefinition || o instanceof Classification || o instanceof Variety) {
                    removeAction.setEnabled(true);
                    break;
                }
            }
        }
    }
}