/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.crop.visual;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.ActionMap;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.TreeTableView;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Visual component to display a treetableview of the croplist with a toolbar on
 * top. Uses explorermanager to get the rootnode.
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Wytse Visser
 */
@NbBundle.Messages({
    "displayName_PROP_NAME=Name",
    "displayName_PROP_SOURCE=Source",
    "displayName_PROP_COLOR=Color"
})
public class CropListVisualPanel extends JPanel implements ExplorerManager.Provider, Lookup.Provider {

    private final ExplorerManager explorerManager;
    private final TreeTableView treeView = new TreeTableView();
    private Lookup lookup;

    public CropListVisualPanel(ExplorerManager explorerManager) {
        this.explorerManager = explorerManager;
        setLayout(new BorderLayout());

        lookup = createLookup();
        add(createScrollPane());
        initData();
    }

    private Lookup createLookup() {
        return ExplorerUtils.createLookup(explorerManager, new ActionMap());
    }

    private JScrollPane createScrollPane() {
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setColumnHeaderView(createToolbar());
        scrollPane.setViewportView(treeView);

        return scrollPane;
    }

    private JToolBar createToolbar() {
        JToolBar toolbar = new CropListToolBar(lookup);
        toolbar.setFloatable(false);
        return toolbar;
    }

    private void initData() {
        try {
            treeView.setProperties(createDummyProperties());
            treeView.setRootVisible(false);
        } catch (NoSuchMethodException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    private static Node.Property[] createDummyProperties() throws NoSuchMethodException {
        final EnterpriseIdProvider EIP = new AgroURI("AGRONL/000000000/CTN/1");
        CropDefinition dummy = new CropDefinition(EIP, new Crop(EIP, "DUMMY_CROP"));

        Node.Property nameProp = new PropertySupport.Reflection(dummy, String.class, CropDefinition.PROP_NAME);
        nameProp.setDisplayName(Bundle.displayName_PROP_NAME());
        nameProp.setValue("ComparableColumnTTV", Boolean.TRUE); //NOI18N
        nameProp.setValue("SortingColumnTTV", Boolean.TRUE); //NOI18N

        Node.Property sourceProp = new PropertySupport.Reflection(dummy, String.class, "getSource", null);
        sourceProp.setDisplayName(Bundle.displayName_PROP_SOURCE());

        Node.Property colorProp = new PropertySupport.Reflection(dummy, Color.class, "getColor", null);
        colorProp.setDisplayName(Bundle.displayName_PROP_COLOR());

        return new Node.Property[]{nameProp, sourceProp, colorProp};
    }

    @Override
    public ExplorerManager getExplorerManager() {
        return explorerManager;
    }

    boolean valid() {
        return true;
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }
}
