/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.crop.visual;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Set;
import javax.swing.JComponent;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropBean;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.CropDefinitionManager;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.stock.crop.storage.PropertyChangeTracker;
import nl.cloudfarming.client.stock.crop.visual.nodes.CropsChildFactory;
import nl.cloudfarming.client.stock.crop.visual.nodes.CropsRootNode;
import org.netbeans.spi.options.OptionsPanelController;
import org.openide.explorer.ExplorerManager;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;

@OptionsPanelController.TopLevelRegistration(
    categoryName = "#OptionsCategory_Name_CropList",
iconBase = "nl/cloudfarming/client/stock/crop/box.png",
keywords = "#OptionsCategory_Keywords_CropList",
keywordsCategory = "Crop List")
@org.openide.util.NbBundle.Messages({"OptionsCategory_Name_CropList=Crop list", "OptionsCategory_Keywords_CropList=Crop list"})
public final class CroplistOptionsPanelController extends OptionsPanelController {

    private CropListVisualPanel panel;
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private boolean changed;
    private PropertyChangeTracker propertyChangeTracker = new PropertyChangeTracker();

    @Override
    public void update() {
        changed = false;
    }

    @Override
    public void applyChanges() {
        Set<CropBean> changedCropBeans = propertyChangeTracker.getUpdatedCropBeans();
        for (CropBean cropBean : changedCropBeans) {
            if (cropBean instanceof CropDefinition) {
                Lookup.getDefault().lookup(CropDefinitionManager.class).save((CropDefinition) cropBean);
            }
            if (cropBean instanceof Crop) {
                Lookup.getDefault().lookup(CropManager.class).save((Crop) cropBean);
            }
        }
        propertyChangeTracker.reset();
        changed = false;
    }

    @Override
    public void cancel() {
        propertyChangeTracker.undoPropertyChanges();
    }

    @Override
    public boolean isValid() {
        return getPanel().valid();
    }

    @Override
    public boolean isChanged() {
        return changed;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.stock.crop.croplist");
    }

    @Override
    public JComponent getComponent(Lookup masterLookup) {
        return getPanel();
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }

    private CropListVisualPanel getPanel() {
        if (panel == null) {
            panel = new CropListVisualPanel(createExplorerManager());
        }
        return panel;
    }

    private ExplorerManager createExplorerManager() {
        CropsRootNode rootNode = new CropsRootNode(new CropsChildFactory(propertyChangeTracker));
        ExplorerManager explorerManager = new ExplorerManager();
        explorerManager.setRootContext(rootNode);

        return explorerManager;
    }

    void changed() {
        if (!changed) {
            changed = true;
            pcs.firePropertyChange(OptionsPanelController.PROP_CHANGED, false, true);
        }
        pcs.firePropertyChange(OptionsPanelController.PROP_VALID, null, null);
    }
}
